/* 
 * File:   EC_I2C.h
 * Author: Jonathan Peterson
 *
 * PIC32MX795 I2C1 Output
 */

#ifndef EC_I2C_H
#define	EC_I2C_H
#include <xc.h>
#include "sourcecode/SDlib16.h"
#include "sourcecode/configbits.h"

/*
 using internal FRC (80 MHz)
 peripheral clock = at 40 MHz (80 MHz/8)
 */



void I2C1_init(unsigned long rate) {
    
    AD1PCFG = 0x1111;
    DDPCONbits.JTAGEN = 0;
    
    I2C1CONbits.ON = 0;
    I2C1BRG = (get_pb_clock()/(2*rate)) - 
            (get_pb_clock()/10000000UL) - 2;
    I2C1CONbits.DISSLW = 1;
    if(rate == 400000ul)I2C1CONbits.DISSLW = 0;
    I2C1CONbits.ON = 1;
    while(!I2C1CONbits.ON)
    return;         
}

void I2C_start(void) {
    IFS0bits.I2C1MIF = 0; // SSP intflag 
    I2C1CONbits.SEN = 1; // send "start bit" 
    while(!IFS0bits.I2C1MIF);  // wait for flag set 
    return; //success
}

void I2C_restart (void) {
    IFS0bits.I2C1MIF = 0; // clear master inter 
    I2C1CONbits.RSEN = 1; // send "start bit" 
    while(!IFS0bits.I2C1MIF);  // wait for flag set 
    return;
}

void I2C_stop (void) {
    IFS0bits.I2C1MIF = 0; // master intflag 
    I2C1CONbits.PEN = 1; 
    while(!IFS0bits.I2C1MIF);  // wait for flag set 
    return;

}

#define i2c_ack_not_recv I2C1STATbits.ACKSTAT  // 1 = ack not recieved
unsigned char I2C_write (unsigned char data) {
    unsigned char result; 
    IFS0bits.I2C1MIF = 0; // SSP intflag 
    I2C1TRN = data; // send the data 
    while(!IFS0bits.I2C1MIF); 
    if (i2c_ack_not_recv) 
        result = -1; // no ack 
    else 
        result = 0; // ack 
    return result;

}

#define i2c_ack  I2C1CONbits.ACKEN  // master send of ackor not 
#define i2c_nack I2C1CONbits.ACKDT  // bit that is sent by ack(1 = NAK!) 
void I2C_ack (void) {
    IFS0bits.I2C1MIF = 0; 
    i2c_ack = 1; // enable acksequence to go 
    while(!IFS0bits.I2C1MIF);  // wait for flag set 
    return;
}

unsigned char I2C_read (unsigned char ack) {
    unsigned char data; 
    IFS0bits.I2C1MIF = 0; // SSP intflag 
    I2C1CONbits.RCEN = 1; // set recieveenable 
    while(!IFS0bits.I2C1MIF);  // wait for flag set 
    data = I2C1RCV; // get data 
    if (ack) 
        i2c_nack = 0; // 0= here means send ack 
    else 
        i2c_nack = 1; // 1 means don't  ack 
    /* do the ack*/ 
    I2C_ack();
    return data;
}



unsigned char I2C_ec_read_byte (unsigned char dev, unsigned char reg) {
    unsigned char ack;
    unsigned char data;
    I2C_start();
    I2C_write(dev<<1);
    I2C_write(reg);
    I2C_restart();
    ack = I2C_write(dev<<1 | 1); 
    data = I2C_read(0);
    if (ack == -1 ){
        return -1; //error
    }
    I2C_stop();
    return data;
}

unsigned char I2C_ec_write_byte (unsigned char dev, unsigned char reg, unsigned char data) {   
    unsigned char ack;
    I2C_start();
    ack = I2C_write(dev<<1);
    I2C_write(reg);
    ack = I2C_write(data);
    if((ack == -1) | (I2C1STATbits.IWCOL))/* If write collision occurs,return -1 */
    {
        I2C_stop();
        return -1; //error
    } 
    else 
    {
        I2C_stop();
        return 0;
    }
}

unsigned char I2C_ec_read_multi (unsigned char dev, unsigned char reg, int read_len, unsigned char *data) {
    unsigned char ack;
    //unsigned char * ptr = data;
    unsigned char result;
    I2C_start();
    ack = I2C_write(dev<<1);
    ack = I2C_write(reg);
    I2C_restart();
    ack = I2C_write(dev<<1 | 1);
    if (ack == -1 ){
    return -1; //error
    }
    int i;
    for (i=1; i < read_len; i++) {
        result = I2C_read(1);
        *data = result;
        data++;
    }
    *data = I2C_read(0);
    I2C_stop();
    return 0;
}

unsigned char I2C_ec_write_multi (unsigned char dev, unsigned char reg, int write_len, unsigned char *data) {
    unsigned char ack;
    I2C_start();
    I2C_write(dev<<1);
    I2C_write(reg);
    int i;
    for (i=1; i < write_len; i++) {
        ack = I2C_write(*data);
        if(ack == -1)
        {    
            return -1;
        }
        else 
        {
            while(I2C1STATbits.TRSTAT); //Wait till data is transmitted.
            data++;
        }
    }
    ack = I2C_write(*data);
    I2C_stop();
    return 0;       
}

#endif	/* EC_I2C_H */